'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var _extends_1 = createCommonjsModule(function (module) {
function _extends() {
  module.exports = _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

module.exports = _extends;
});

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

const roundUp = num => Number(num.toFixed(2));

const isNaN = num => num !== num;

class Point {
  constructor(x, y) {
    this.x = x;
    this.y = y;
  }

  toVector() {
    const value = Math.sqrt(Math.pow(this.x, 2.0) + Math.pow(this.y, 2.0));
    const angle = Math.atan2(this.y, this.x);
    return new Vector(value, angle);
  }

  scale(r) {
    return new Point(this.x * r, this.y * r);
  }

  add(p) {
    return new Point(this.x + p.x, this.y + p.y);
  }

  sub(p) {
    return new Point(this.x - p.x, this.y - p.y);
  }

  eql(p) {
    return this.x === p.x && this.y === p.y;
  }

  clone() {
    return new Point(this.x, this.y);
  }

  toString() {
    return this.x + " " + this.y;
  }

}
let CommandType; // TODO: compatible CommandType

(function (CommandType) {
  CommandType["MOVE"] = "M";
  CommandType["LINE"] = "L";
  CommandType["CURVE"] = "C";
  CommandType["CLOSE"] = "Z";
})(CommandType || (CommandType = {}));

class Command {
  // TODO: Convert data format to number array.
  constructor(type, point) {
    this.point = point;
    this.type = type;
  }

  toString() {
    switch (this.type) {
      case CommandType.MOVE:
      case CommandType.LINE:
        return this.type + " " + this.point.toString();

      case CommandType.CURVE:
        if (!this.cl || !this.cr) return CommandType.LINE + " " + this.point.x + " " + this.point.y;
        return this.type + " " + this.cl.toString() + " " + this.cr.toString() + " " + this.point.toString();

      default:
        return '';
    }
  }

  scale(r) {
    var _this$cr, _this$cl;

    const upd = new Command(this.type, this.point.scale(r));
    upd.cr = (_this$cr = this.cr) == null ? void 0 : _this$cr.scale(r);
    upd.cl = (_this$cl = this.cl) == null ? void 0 : _this$cl.scale(r);
    return upd;
  }

  clone() {
    var _this$cl2, _this$cr2;

    const copy = new Command(this.type, this.point.clone());
    copy.cl = (_this$cl2 = this.cl) == null ? void 0 : _this$cl2.clone();
    copy.cr = (_this$cr2 = this.cr) == null ? void 0 : _this$cr2.clone();
    return copy;
  }

}
class Vector {
  constructor(v, a) {
    this.value = v;
    this.angle = a;
  }

  toPoint() {
    const x = Math.cos(this.angle) * this.value;
    const y = Math.sin(this.angle) * this.value;
    return new Point(roundUp(x), roundUp(y));
  }

  scale(r) {
    return new Vector(this.value * r, this.angle);
  }

}

/**
 * TODO: refactor command.
 * The following commands are not supported. Cannot support commands that use `M` or` z` more than once
 * `M 0 0 L 1 1 Z M 1 1 L 2 2 Z`
 */
class Path {
  constructor({
    close,
    curve,
    strokeWidth,
    stroke,
    fill
  } = {}) {
    this.close = close != null ? close : false;
    this.curve = curve != null ? curve : true;
    this.strokeWidth = strokeWidth != null ? strokeWidth : 1;
    this.stroke = stroke != null ? stroke : '#000';
    this.fill = fill != null ? fill : 'none';
    this.smoothRatio = 0.2;
    this.commands = [];
  }

  scale(r) {
    this.commands = this.commands.map(c => c.scale(r));
    this.strokeWidth *= r;
    return this;
  }

  addCommand(param) {
    if (param instanceof Point) {
      if (!this.curve || this.commands.length < 2) {
        this.commands.push(new Command(this.commands.length === 0 ? CommandType.MOVE : CommandType.LINE, param));
      } else {
        const p1 = this.commands.length === 2 ? this.commands[this.commands.length - 2].point : this.commands[this.commands.length - 3].point;
        const p2 = this.commands[this.commands.length - 2].point;
        const p3 = this.commands[this.commands.length - 1].point;
        const p4 = param;
        this.commands[this.commands.length - 1] = this._createBezierCurveCommand(p1, p2, p3, p4);
        this.commands.push(this._createBezierCurveCommand(p2, p3, p4, p4));
      }
    } else {
      this.commands.push(param);
    }

    return this;
  }

  getCommandString() {
    if (this.commands.length === 0) return '';
    let d = this.commands.map((com, _i) => com.toString());

    if (this.close) {
      d.push(CommandType.CLOSE);
    }

    return d.join(' ');
  } // TODO: Increase supported command types
  // TODO: Comma parse


  parseCommandString(d) {
    this.commands = [];
    const c = d.split(' ');

    if (c[c.length - 1] === CommandType.CLOSE) {
      this.close = true;
    }

    for (let i = 0; i < c.length; i += 1) {
      switch (c[i]) {
        case CommandType.MOVE:
        case CommandType.LINE:
          {
            if (i + 2 > c.length) return;
            const p1 = Number(c[i + 1]);
            const p2 = Number(c[i + 2]);
            if (isNaN(p1) || isNaN(p2)) return; // check NaN

            this.commands.push(new Command(c[i], new Point(p1, p2)));
            i += 2;
            break;
          }

        case CommandType.CURVE:
          {
            if (i + 6 > c.length) return;
            const p1 = Number(c[i + 1]);
            const p2 = Number(c[i + 2]);
            const p3 = Number(c[i + 3]);
            const p4 = Number(c[i + 4]);
            const p5 = Number(c[i + 5]);
            const p6 = Number(c[i + 6]);
            if (isNaN(p1) || isNaN(p2) || isNaN(p3) || isNaN(p4) || isNaN(p5) || isNaN(p6)) return; // check NaN

            const curveComand = new Command(c[i], new Point(p5, p6));
            curveComand.cl = new Point(p1, p2);
            curveComand.cr = new Point(p3, p4);
            this.commands.push(curveComand);
            i += 6;
            break;
          }
      }
    }
  }

  parsePathElement(pEl) {
    const d = pEl.getAttribute('d');

    if (d) {
      this.parseCommandString(d);
    }

    const strokeWidth = pEl.getAttribute('stroke-width');

    if (strokeWidth) {
      this.strokeWidth = Number(strokeWidth);
    }

    const stroke = pEl.getAttribute('stroke');

    if (stroke) {
      this.stroke = stroke;
    }

    const fill = pEl.getAttribute('fill');

    if (fill) {
      this.fill = fill;
    }

    return this;
  }

  toJson() {
    return {
      fill: this.fill,
      d: this.getCommandString(),
      stroke: this.stroke,
      strokeWidth: this.strokeWidth,
      strokeLinecap: this.curve ? 'round' : 'square',
      strokeLinejoin: this.curve ? 'round' : 'mitter'
    };
  }

  toElement() {
    const path = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    path.setAttribute('stroke-width', String(this.strokeWidth));
    path.setAttribute('stroke', this.stroke);
    path.setAttribute('fill', this.fill);
    path.setAttribute('stroke-linejoin', this.curve ? 'round' : 'mitter');
    path.setAttribute('stroke-linecap', this.curve ? 'round' : 'square');
    path.setAttribute('d', this.getCommandString());
    return path;
  }

  clone() {
    const path = new Path({
      close: this.close,
      curve: this.curve,
      strokeWidth: this.strokeWidth,
      stroke: this.stroke,
      fill: this.fill
    });
    this.commands.map(c => {
      path.addCommand(c.clone());
    });
    return path;
  }

  _createBezierCurveCommand(p1, p2, p3, p4) {
    const cmd = new Command(CommandType.CURVE, p3);
    cmd.cl = this._createControlPoint(p1, p2, p3);
    cmd.cr = this._createControlPoint(p4, p3, p2);
    return cmd;
  }

  _createControlPoint(prev, start, next) {
    const contolVectorPoint = next.sub(prev).toVector().scale(this.smoothRatio).toPoint();
    return start.add(contolVectorPoint);
  }

}
class Svg {
  constructor({
    width,
    height
  }) {
    this._paths = [];
    this.width = width;
    this.height = height;
  }

  scalePath(r) {
    if (r !== 1) {
      for (let i = 0; i < this._paths.length; i += 1) {
        this._paths[i].scale(r);
      }
    }

    return this;
  }

  addPath(pa) {
    this._paths.push(pa);

    return this;
  }

  undoPath() {
    return this._paths.pop();
  }

  replacePaths(paths) {
    this._paths = paths;
    return this;
  }

  clonePaths() {
    return this._paths.map(p => p.clone());
  }

  addCommand(po) {
    if (this._paths.length === 0) return this;
    const updateIndex = this._paths.length - 1;

    this._paths[updateIndex].addCommand(po);

    return this;
  }

  updatePath(pa, i) {
    const updateIndex = i || this._paths.length - 1;
    if (updateIndex < 0) this._paths.push(pa);
    this._paths[updateIndex] = pa;
    return this;
  }

  clearPath() {
    this._paths = [];
    return this;
  }

  get paths() {
    return this._paths;
  }

  toJson() {
    return {
      width: this.width,
      height: this.height,
      paths: this.paths.map(p => p.toJson())
    };
  }

  copy(svg) {
    this._paths = svg.clonePaths();
    this.scalePath(svg.width / this.width);
    return this;
  }

  toElement() {
    const svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
    svg.setAttribute('width', String(this.width));
    svg.setAttribute('height', String(this.height));

    for (let i = 0; i < this._paths.length; i += 1) {
      svg.appendChild(this._paths[i].toElement());
    }

    return svg;
  }

  toBase64() {
    const data = "<svg width=\"" + this.width + "\" height=\"" + this.height + "\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">" + this.toElement().innerHTML + "</svg>";
    return "data:image/svg+xml;base64," + btoa(data);
  }

  parseSVGString(svgStr) {
    const svgEl = new DOMParser().parseFromString(svgStr, 'image/svg+xml').querySelector('svg');
    return !svgEl ? this.clearPath() : this.parseSVGElement(svgEl);
  }

  parseSVGElement(svgEl) {
    const update = [];
    svgEl.querySelectorAll('path').forEach(pEl => {
      const pa = new Path().parsePathElement(pEl);

      if (pa.commands.length !== 0) {
        update.push(pa);
      }
    });
    this.replacePaths(update);
    const width = Number(svgEl.getAttribute('width'));

    if (width) {
      this.scalePath(this.width / width);
    }

    return this;
  }

}

const mimeTypeMap = {
  png: 'image/png',
  jpg: 'image/jpeg',
  svg: 'image/svg+xml'
};
const download = ({
  data,
  extension,
  filename
}) => {
  const bin = atob(data.replace(/^.*,/, ''));
  const buffer = new Uint8Array(bin.length);

  for (let i = 0; i < bin.length; i += 1) {
    buffer[i] = bin.charCodeAt(i);
  }

  const fname = filename || Date.now() + "." + extension;
  const blob = new Blob([buffer.buffer], {
    type: mimeTypeMap[extension]
  });

  if (window.navigator.msSaveBlob) {
    // IE
    window.navigator.msSaveBlob(blob, fname);
  } else if (window.URL && window.URL.createObjectURL) {
    // Firefox, Chrome, Safari
    const a = document.createElement('a');
    a.download = fname;
    a.href = window.URL.createObjectURL(blob);
    document.body.appendChild(a);
    a.click();
    document.body.removeChild(a);
  } else {
    // Other
    window.open(data, '_blank');
  }
};

class Renderer extends Svg {
  constructor(el, {
    background
  } = {}) {
    const {
      width,
      height,
      left,
      top
    } = el.getBoundingClientRect();
    super({
      width,
      height
    });
    /**
     * Setup parameter
     */

    this.el = el;
    this.left = left;
    this.top = top;
    this.background = background != null ? background : '#fff';
    el.appendChild(this.toElement());

    this._setupAdjustResize();
  }
  /**
   * render
   * TODO: XSS test
   */


  update() {
    this.el.replaceChild(this.toElement(), this.el.childNodes[0]);
  }

  resizeElement(param) {
    const {
      width,
      height,
      left,
      top
    } = param || this.el.getBoundingClientRect(); // TODO: Resizing improve

    this.scalePath(width / this.width);
    this.width = width;
    this.height = height;
    this.left = left;
    this.top = top;
  }

  _setupAdjustResize() {
    if (window.ResizeObserver) {
      const resizeObserver = new window.ResizeObserver(entries => {
        this.resizeElement(entries[0].contentRect);
        this.update();
      });
      resizeObserver.observe(this.el);
    } else {
      // TODO: improve
      window.addEventListener('resize', _ev => {
        this.resizeElement(this.el.getBoundingClientRect());
        this.update();
      });
    }
  } // TODO: Add filename config


  download(ext = 'svg', cb = download) {
    if (ext === 'svg') {
      cb({
        data: this.toBase64(),
        extension: 'svg'
      });
    }

    const img = new Image();

    const renderCanvas = () => {
      const canvas = document.createElement('canvas');
      canvas.setAttribute('width', String(this.width));
      canvas.setAttribute('height', String(this.height));
      const ctx = canvas.getContext('2d');
      if (!ctx) return;
      ctx.fillStyle = this.background;
      ctx.fillRect(0, 0, this.width, this.height);
      ctx.drawImage(img, 0, 0);

      if (ext === 'png') {
        cb({
          data: canvas.toDataURL('image/png'),
          extension: 'png'
        });
      } else {
        cb({
          data: canvas.toDataURL('image/jpeg'),
          extension: 'jpg'
        });
      }
    };

    img.addEventListener('load', renderCanvas, false);
    img.src = this.toBase64();
  }

}

function throttle(func, wait, options = {}) {
  let context;
  let args;
  let result;
  let timeout = null;
  let previous = 0;

  const later = () => {
    previous = options.leading === false ? 0 : Date.now();
    timeout = null;
    result = func.apply(context, args);

    if (!timeout) {
      context = null;
      args = null;
    }
  };

  return function wrap(...wraparg) {
    const now = Date.now();
    if (!previous && options.leading === false) previous = now;
    const remaining = wait - (now - previous);
    context = this;
    args = wraparg;

    if (remaining <= 0 || remaining > wait) {
      if (timeout) {
        clearTimeout(timeout);
        timeout = null;
      }

      previous = now;
      result = func.apply(context, args);

      if (!timeout) {
        context = null;
        args = null;
      }
    } else if (!timeout && options.trailing !== false) {
      timeout = setTimeout(later, remaining);
    }

    return result;
  };
}

const getPassiveOptions = (passive = true) => {
  try {
    const check = () => null;

    window.addEventListener('testPassive', check, {
      passive
    });
    window.removeEventListener('testPassive', check);
    return {
      passive
    };
  } catch (e) {
    return false;
  }
};

class SvgDrawing extends Renderer {
  constructor(el, _ref = {}) {
    let {
      penColor,
      penWidth,
      curve,
      close,
      delay,
      fill
    } = _ref,
        rendOpt = _objectWithoutPropertiesLoose(_ref, ["penColor", "penWidth", "curve", "close", "delay", "fill"]);

    super(el, _extends({}, rendOpt));
    /**
     * Setup parameter
     */

    this.penColor = penColor != null ? penColor : '#333';
    this.penWidth = penWidth != null ? penWidth : 1;
    this.curve = curve != null ? curve : true;
    this.close = close != null ? close : false;
    this.delay = delay != null ? delay : 20;
    this.fill = fill != null ? fill : 'none';
    this._drawPath = null;
    this._clearPointListener = null;
    this._clearMouseListener = null;
    this._clearTouchListener = null;
    this._drawingMove = this._drawingMove.bind(this); // for throttle

    this.on();
  }

  clear() {
    this.clearPath();
    this.update();
  }

  undo() {
    this.undoPath();
    this.update();
  }

  changeDelay(delay) {
    this.delay = delay;
    this.on();
  }

  on() {
    this.off();

    if (window.PointerEvent) {
      this._setupPointEventListener();
    } else {
      this._setupMouseEventListener();
    }

    if ('ontouchstart' in window) {
      this._setupTouchEventListener();
    }
  }

  off() {
    if (this._clearPointListener) {
      this._clearPointListener();

      this._clearPointListener = null;
    }

    if (this._clearMouseListener) {
      this._clearMouseListener();

      this._clearMouseListener = null;
    }

    if (this._clearTouchListener) {
      this._clearTouchListener();

      this._clearTouchListener = null;
    }
  }

  _drawingStart() {
    this._drawPath = this._createDrawPath();
    this.addPath(this._drawPath);
  }

  _drawingMove({
    x,
    y
  }) {
    const po = this._createDrawPoint({
      x,
      y
    });

    this.addCommand(po);

    if (!this._drawPath || this._drawPath.strokeWidth !== this.penWidth || this._drawPath.stroke !== this.penColor) {
      this._drawPath = this._createDrawPath().addCommand(po);
      this.addPath(this._drawPath);
    }

    this.update();
  }

  _createDrawPath() {
    this.resizeElement();
    return new Path({
      close: this.close,
      curve: this.curve,
      stroke: this.penColor,
      strokeWidth: this.penWidth,
      fill: this.fill
    });
  }

  _createDrawPoint({
    x,
    y
  }) {
    return new Point(x - this.left, y - this.top);
  }
  /**
   * Drawing MouseEvent
   */


  _setupPointEventListener() {
    const handleMouse = cb => ev => {
      ev.preventDefault();
      cb({
        x: ev.clientX,
        y: ev.clientY
      });
    };

    const start = handleMouse(param => {
      this._drawingStart();

      this.el.addEventListener('pointermove', draw, getPassiveOptions(false));
      this.el.addEventListener('pointerup', end, getPassiveOptions(false));
      this.el.addEventListener('pointerleave', end, getPassiveOptions(false));
      this.el.addEventListener('pointercancel', end, getPassiveOptions(false));
    });
    const draw = throttle(handleMouse(this._drawingMove), this.delay);
    const end = handleMouse(param => {
      this.el.removeEventListener('pointermove', draw);
      this.el.removeEventListener('pointerup', end);
      this.el.removeEventListener('pointerleave', end);
      this.el.addEventListener('pointercancel', end);
    });
    this.el.addEventListener('pointerdown', start, getPassiveOptions(false));

    this._clearPointListener = () => this.el.removeEventListener('pointerdown', start);
  }
  /**
   * Drawing MouseEvent
   */


  _setupMouseEventListener() {
    const handleMouse = cb => ev => {
      ev.preventDefault();
      cb({
        x: ev.clientX,
        y: ev.clientY
      });
    };

    const start = handleMouse(_param => {
      this._drawingStart();

      this.el.addEventListener('mousemove', draw, getPassiveOptions(false));
      this.el.addEventListener('mouseup', end, getPassiveOptions(false));
      this.el.addEventListener('mouseleave', end, getPassiveOptions(false));
    });
    const draw = throttle(handleMouse(this._drawingMove), this.delay);
    const end = handleMouse(_param => {
      this.el.removeEventListener('mousemove', draw);
      this.el.removeEventListener('mouseup', end);
      this.el.removeEventListener('mouseleave', end);
    });
    this.el.addEventListener('mousedown', start, getPassiveOptions(false));

    this._clearMouseListener = () => this.el.removeEventListener('mousedown', start);
  }
  /**
   * Drawing TouchEvent
   */


  _setupTouchEventListener() {
    const handleTouch = cb => ev => {
      ev.preventDefault();
      const touch = ev.touches[0];
      cb({
        x: touch.clientX,
        y: touch.clientY
      });
    };

    const start = handleTouch(_param => {
      this._drawingStart();

      this.el.addEventListener('touchmove', draw, getPassiveOptions(false));
      this.el.addEventListener('touchend', end, getPassiveOptions(false));
    });
    const draw = throttle(handleTouch(this._drawingMove), this.delay);
    const end = handleTouch(_param => {
      this.el.removeEventListener('touchmove', draw);
      this.el.removeEventListener('touchend', end);
    });
    this.el.addEventListener('touchstart', start, getPassiveOptions(false));

    this._clearTouchListener = () => this.el.removeEventListener('touchstart', start);
  }

}

var useSvgDrawing = function useSvgDrawing(option) {
  var renderRef = react.useRef(null);
  var drawingRef = react.useRef(null);
  var getSvgXML = react.useCallback(function () {
    if (!drawingRef.current) return null;
    return drawingRef.current.toElement().outerHTML;
  }, []);
  var download = react.useCallback(function (ext) {
    if (ext === void 0) {
      ext = 'svg';
    }

    if (!drawingRef.current) return;
    drawingRef.current.download(ext);
  }, []);
  var changePenColor = react.useCallback(function (param) {
    if (!drawingRef.current || !param) return;
    drawingRef.current.penColor = param;
  }, []);
  var changeFill = react.useCallback(function (param) {
    if (!drawingRef.current || !param) return;
    drawingRef.current.fill = param;
  }, []);
  var changeDelay = react.useCallback(function (param) {
    if (!drawingRef.current || !param) return;
    drawingRef.current.delay = param;
  }, []);
  var changePenWidth = react.useCallback(function (param) {
    if (!drawingRef.current) return;
    drawingRef.current.penWidth = Number(param);
  }, []);
  var changeClose = react.useCallback(function (param) {
    if (!drawingRef.current || !param) return;
    drawingRef.current.close = param;
  }, []);
  var changeCurve = react.useCallback(function (param) {
    if (!drawingRef.current || !param) return;
    drawingRef.current.curve = param;
  }, []);
  var clear = react.useCallback(function () {
    if (!drawingRef.current) return;
    drawingRef.current.clear();
  }, []);
  var undo = react.useCallback(function () {
    if (!drawingRef.current) return;
    drawingRef.current.undo();
  }, []);
  react.useEffect(function () {
    if (drawingRef.current) return;
    if (!renderRef.current) return;
    drawingRef.current = new SvgDrawing(renderRef.current, _extends_1({}, option));
  });
  return [renderRef, {
    instance: drawingRef.current,
    changePenWidth: changePenWidth,
    changePenColor: changePenColor,
    changeFill: changeFill,
    changeDelay: changeDelay,
    changeClose: changeClose,
    changeCurve: changeCurve,
    clear: clear,
    undo: undo,
    getSvgXML: getSvgXML,
    download: download
  }];
};

exports.useSvgDrawing = useSvgDrawing;
//# sourceMappingURL=index.cjs.js.map
